/* Stats - IRC Statistical Services
**
** SplitServ functionality
**
** $Id: splitserv.c,v 1.18 2001/09/03 22:48:00 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "help.h"
#include "patchlevel.h"
#include "text.h"

#ifdef SPLITSERV

struct SplTable
{
        char* name;
        long counter;
	int channel;  /* 1 if it's allowed in a channel */
        int (*handler)();
};

extern const char version_date[], version_time[];
static char errstr[BUFSIZE];

int sp_map(char *, char *, int, char **);
int sp_split(char *, char *, int, char **);
int sp_dump(char *, char *, int, char **);
int sp_add(char *, char *, int, char **);
int sp_forget(char *, char *, int, char **);
int sp_update(char *, char *, int, char **);
int sp_help(char *, char *, int, char **);
int sp_chelp(char *, char *, int, char **);
int sp_cping(char *, char *, int, char **);
int sp_cversion(char *, char *, int, char **);
int sp_triggers(char *, char *, int, char **);

static struct SplTable sp_table[] =
{
	{ TRIGGER_SPLIT,0,      1,      sp_split },
	{ TRIGGER_MAP,  0,      1,      sp_map },
        { "MAP",        0,      0,      sp_map },
        { "SPLIT",      0,      0,      sp_split },
        { TRIGGER_DUMP, 0,      1,      sp_dump },
        { "DUMP",       0,      0,      sp_dump },
        { "FORGET",     0,      0,      sp_forget },
        { "ADD",        0,      0,      sp_add },
        { "UPDATE",     0,      0,      sp_update },
	{ "HELP",	0,	0,	sp_help },
	{ TRIGGER_HELP,	0,	1,	sp_chelp },
	{ TRIGGER_TRIGGERS,0,	1,	sp_triggers },
	{ "TRIGGERS",	0,	0,	sp_triggers },
	{ "\001PING",	0,	1,	sp_cping },
	{ "\001VERSION",0,	1,	sp_cversion },
	{ (char *)0,	0,	0,	(int (*)()) 0 }
};

static struct HelpTable help_table[] =
{
        { TRIGGER_MAP,  sp_map_help,            0 },
        { "MAP",        sp_map_help,            0 },
	{ TRIGGER_SPLIT,sp_split_help,		0 },
	{ "SPLIT",	sp_split_help,		0 },
	{ TRIGGER_DUMP,	sp_dump_help,		0 },
	{ "DUMP",	sp_dump_help,		0 },
	{ "FORGET",	sp_forget_help,		1 },
	{ "ADD",	sp_add_help,		1 },
	{ "UPDATE",	sp_update_help,		1 },
        { (char *)0,    (const char **)0,       0 }
};


void splitserv(char *origin, char *target, char *param)
{
	char *parv[MAXPARA];
	int parc = 0;
	User *u = NULL;
	int i;
	int status;
	static char buf[BUFSIZE];

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

	/* Clear the error string */
	errstr[0] = '\0';

	strncpy(buf, param, BUFSIZE);

	if (param)
		parc = msgtoken(param, parv);

	if (origin)
		u = finduser(origin);
	else
		return;

        if (!u)
	{
                log("Unable to finduser %s (splitserv)", origin);
                return;
        }

	netinfo.requests++;

	if (!parc)
		return;

	if (!strcasecmp(target, sp.join_chan) &&
		(isalpha(parv[0][0]) || isdigit(parv[0][0])))
		return;

	if (parv[0][0] != '\001')
		log("%s received message from %s to %s: %s", SplitServ.nick, 
			origin, target, buf);

	/* All the preliminary junk is done, now we have to go through 
	** the tables and call the handler for that command
	*/
        for (i=0; sp_table[i].handler; i++)
        {
                if (!strcasecmp(parv[0], sp_table[i].name))
                {
		  if (is_channel(target) && !sp_table[i].channel)
			return;
                    status = sp_table[i].handler(u->nick, target, parc, parv);
                    if (status == 1)
                        splog("Error with splitserv command %s: %s", parv[0],
			(errstr) ? errstr : "No error message");
		    /* Matched a value, done, then return */
		    return;
                }
        }
	/* If we get here, we did not get a valid command...
	** Make sure not to say anything to the user if it was a 
	** CTCP that was not matched
	*/
	if (*parv[0] == '\001')
		return;
	/* Don't answer invalid commands if it was a message to a channel */
	if (is_channel(target))
		return;
	privmsg(u->nick, SplitServ.nick, "Invalid command: %s", parv[0]);
	return;
}

/* In all of these functions, parv[0] contains the command, followed
** by the parameters
*/

int sp_chelp(char *origin, char *target, int parc, char *parv[])
{
	spprivmsg_list(sp.join_chan, SplitServ.nick, sp_help_pub);
	return 0;
}

int sp_help(char *origin, char *target, int parc, char *parv[])
{
	User *u = finduser(origin);
	int i;

	if (!parv[1])
	{
		spprivmsg_list(u->nick, SplitServ.nick, sp_help_main);
		return 0;
	}

	for (i = 0; help_table[i].helpmsg; i++)
	{
		if (!strcasecmp(parv[1], help_table[i].name))
                {
			if (help_table[i].myuonly)
			{
				if (u->myuser)
					spprivmsg_list(u->nick, SplitServ.nick,
						help_table[i].helpmsg);
				else
					spprivmsg(u->nick, SplitServ.nick,
						"Permission Denied.");
			}
			else
			{
				spprivmsg_list(u->nick, SplitServ.nick,
					help_table[i].helpmsg);
			}

			return 0;
		}
	}

	spprivmsg(u->nick, SplitServ.nick, "Unknown Help Topic: \2%s\2",
		parv[1]);
	snprintf(errstr, BUFSIZE, "Unknown HELP topic: %s", parv[1]);
	return 1;
}

int sp_map(char *origin, char *target, int parc, char *parv[])
{
	if (is_channel(target))
		PrintMap(target, map_head, 0);
	else
		PrintMap(origin, map_head, 0);
	return 0;
}

int sp_split(char *origin, char *target, int parc, char *parv[])
{
	if (is_channel(target))
		PrintSplit(target);
	else
		PrintSplit(origin);
	return 0;
}

int sp_dump(char *origin, char *target, int parc, char *parv[])
{
	if (is_channel(target))
		PrintDump(target);
	else
		PrintDump(origin);
	return 0;
}

int sp_forget(char *origin, char *target, int parc, char *parv[])
{
	User *u = finduser(origin);

	if (u)
	{
		if (u->myuser)
		{
			if (!strcasecmp(target, SplitServ.nick))
			{
				Forget(origin, parv[1]);
			}
		}
	}
	return 0;
}

int sp_add(char *origin, char *target, int parc, char *parv[])
{
	User *u = finduser(origin);

        if (u)
	{
                if (u->myuser)
		{
                        if (!strcasecmp(target, SplitServ.nick))
			{
                                Add(origin, parv[1]);
                        }
                }
        }
        return 0;
}

int sp_update(char *origin, char *target, int parc, char *parv[])
{
        User *u = finduser(origin);

        if (u)
	{
                if (u->myuser)
		{
                        if (!strcasecmp(target, SplitServ.nick))
			{
                                Update(origin);
                        }
                }
        }
        return 0;
}

int sp_cversion(char *origin, char *target, int parc, char *parv[])
{
        User *u = finduser(origin);

        sts(":%s NOTICE %s :\001VERSION %s(%s) %s %s -> %s %s\001",
	     SplitServ.nick, u->nick, PATCHLEVEL, SERIALNUM, settings.name,
	     VersionFlags(), version_date, version_time);
        splog("CTCP VERSION from %s!%s@%s", u->nick, u->username, u->hostname);

        return 0;
}

int sp_cping(char *origin, char *target, int parc, char *parv[])
{
        User *u = finduser(origin);
        char *command = parv[1];

        if (!command)
                return 1;
        sts(":%s NOTICE %s :\001PING %s", SplitServ.nick, u->nick, command);
        splog("CTCP PING from %s!%s@%s", u->nick, u->username, u->hostname);

        return 0;
}

int sp_triggers(char *origin, char *target, int parc, char *parv[])
{
	char *command = parv[0];

	if (!command)
		return 1;
        if (is_channel(target))
		spprivmsg(target, SplitServ.nick, "Public Triggers: %s %s %s",
			TRIGGER_SPLIT, TRIGGER_MAP, TRIGGER_DUMP);
        else
		spprivmsg(origin, SplitServ.nick, "Public Triggers: %s %s %s",
			TRIGGER_SPLIT, TRIGGER_MAP, TRIGGER_DUMP);
        return 0;
}

#endif
